//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include <fstream.h>
#include <FileCtrl.hpp>
#include <stdio.h>
#include <stdlib.h>
#include <dir.h>
#include <string.h>
#include "MainUnit.h"
#include "About.h"
#include "PieChartOptionsUnit.h"
#include "SplashUnit.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMainForm *MainForm;

bool Altered = false;
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCreate(TObject *Sender)
{
   PageControl1->ActivePage = DataSheet;
   PageControl1Change(Sender);
   c = 1.5;
   beta = 0.7785;
   RowEditChange(Sender);          // sets up the default number of rows
   AnalysesEditChange(Sender);     // sets up the default number of columns
   Memo1->Lines->Clear();          // clear the results page
   Series1->Clear();               // clear the graph page

   ProgramDir = GetCurrentDir();   // remember where you are
   ForceDirectories(ProgramDir+"\\Results");       // always make sure there is a results directory
   ForceDirectories(ProgramDir+"\\Projects");      // always make sure there is a data directory
   if (FileExists(ProgramDir+"\\Ncl.avi"))
   {
      NclAnimate->FileName = ProgramDir+"\\Ncl.avi";
      NclAnimate->Active = true;
   }
   Altered = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
   SetUpArraySizes(0,0,0);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::RowEditChange(TObject *Sender)
{
   // sets number of rows, allows one extra row for titles

   DataGrid->RowCount = RowUpDown->Position + 1;
   DataGrid->Cells[0][0] = "Site Ref.";

   /*for (int i=1; i<DataGrid->RowCount; i++)
   {
      DataGrid->Cells[0][i] = "     "+ IntToStr(i);    // numbers the rows, column[0], row[i]
   }    */

   // now sort out the memory

   SetUpArraySizes(RowUpDown->Position, SampleUpDown->Position, AnalysesUpDown->Position);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AnalysesEditChange(TObject *Sender)
{
   // sets number of columns, allows one extra row for titles

   DataGrid->ColCount = AnalysesUpDown->Position * SampleUpDown->Position + 1;

   for (int i=1; i<DataGrid->ColCount; i++)
   {
      DataGrid->Cells[i][0] = "     "+ IntToStr(i);   // numbers the columns, column[i], row[0]
   }

   // now sort out the memory

   SetUpArraySizes(RowUpDown->Position, SampleUpDown->Position, AnalysesUpDown->Position);
}
//---------------------------------------------------------------------------
void TMainForm::SetUpArraySizes(int X, int Y, int Z)
{
   nlabs = X;    // number of rows
   nbatch = Y;   // number of samples
   nreps = Z;    // number of analyses

   // remember the old lengths of the arrays

   int OldX;
   int OldY;
   int OldZ;

   OldX = Data.Length;   // 3D
   if (OldX > 0)   // make sure there is something to check first
   {
      OldY = Data[0].Length;
      if (OldY > 0)   // make sure there is something to check first
      {
         OldZ = Data[0][0].Length;
      }
   }

   OldX = WRep.Length;   // 3D
   if (OldX > 0)
   {
      OldY = WRep[0].Length;
      if (OldY > 0)
      {
         OldZ = WRep[0][0].Length;
      }
   }

   OldX = Batch.Length;   // 2D
   if (OldX > 0)
   {
      OldZ = Batch[0].Length;
   }

   OldX = WBatch.Length;   // 2D
   if (OldX > 0)
   {
      OldY = WBatch[0].Length;
   }

   OldX = Labs.Length;    // 1D
   OldX = WLabs.Length;   // 1D
   

   // now allocate the size of the arrays

   Data.Length = X;          // 3D
   for (int i=0; i<Data.Length; i++)
   {
      Data[i].Length = Y;
      for (int j=0; j<Data[i].Length; j++)
      {
         Data[i][j].Length = Z;
      }
   }

   WRep.Length = X;          // 3D
   for (int i=0; i<WRep.Length; i++)
   {
      WRep[i].Length = Y;
      for (int j=0; j<WRep[i].Length; j++)
      {
         WRep[i][j].Length = Z;
      }
   }

   Batch.Length = X;         // 2D
   for (int i=0; i<Batch.Length; i++)
   {
      Batch[i].Length = Z;
   }

   WBatch.Length = X;        // 2D
   for (int i=0; i<WBatch.Length; i++)
   {
      WBatch[i].Length = Y;
   }

   Labs.Length = X;          // 1D
   WLabs.Length = X;         // 1D
   Sites.Length = X;         // 1D

   // now make sure if the sizes of the arrays get bigger they are filled
   // with zeros to remove the default wrong values given by the computer

   if ((X > 0) && (Y > 0) && (Z > 0))   // have the arrays got size
   {
      if ( OldX < X )   // anything extra in the X direction, only do it if arrays bigger
      {
         // 1D arrays first

         for (int i=OldX+1; i<Data.High; i++)   // anything extra in the X direction
         {
            Labs[i] = 0;   // set the default value to 0
            WLabs[i] = 0;  // set the default value to 0
         }

         // 2D arrays

         for (int i=OldX+1; i<Batch.High; i++)   // anything extra in the X direction
         {
            for (int j=0; j<Batch[i].High; j++)   // everything in the Z direction
            {
               Batch[i][j] = 0;   // set the default value to 0
            }
         }

         for (int i=OldX+1; i<WBatch.High; i++)   // anything extra in the X direction
         {
            for (int j=0; j<WBatch[i].High; j++)   // everything in the Z direction
            {
               WBatch[i][j] = 0;   // set the default value to 0
            }
         }

         // 3D arrays

         for (int i=OldX+1; i<Data.High; i++)   // anything extra in the X direction
         {
            for (int j=0; j<Data[i].High; j++)   // everything in the Y direction
            {
               for (int k=0; k<Data[i][j].High; k++)   // everything in the Z direction
               {
                  Data[i][j][k] = 0;   // set the default value to 0
               }
            }
         }

         for (int i=OldX+1; i<WRep.High; i++)   // anything extra in the X direction
         {
            for (int j=0; j<WRep[i].High; j++)   // everything in the Y direction
            {
               for (int k=0; k<WRep[i][j].High; k++)   // everything in the Z direction
               {
                   WRep[i][j][k] = 0;   // set the default value to 0
               }
            }
         }
      }

     

      if ( OldY < Y)   // anything extra in the Y direction, only do it if arrays bigger
      {
         // 2D arrays

            for (int i=0; i<WBatch.High; i++)   // everything in the X direction
            {
               for (int j=OldY+1; j<WBatch[i].High; j++)   // anything extra in the Y direction
               {
                  WBatch[i][j] = 0;   // set the default value to 0
               }
            }

         // 3D arrays

         for (int i=0; i<Data.High; i++)   // everything in the X direction
         {
            for (int j=OldY+1; j<Data[i].High; j++)   // anything extra in the Y direction
            {
               for (int k=0; k<Data[i][j].High; k++)   // everything in the Z direction
               {
                  Data[i][j][k] = 0;   // set the default value to 0
               }
            }
         }

         for (int i=0; i<WRep.High; i++)   // everything in the X direction
         {
            for (int j=OldY+1; j<WRep[i].High; j++)   // anything extra in the Y direction
            {
               for (int k=0; k<WRep[i][j].High; k++)   // everything in the Z direction
               {
                  WRep[i][j][k] = 0;   // set the default value to 0
               }
            }
         }
      }


      if ( OldZ < Z)   // anything extra in the Z direction, only do it if arrays bigger
      {
         // 2D arrays

         for (int i=0; i<Batch.High; i++)   // everything in the X direction
         {
            for (int j=OldZ+1; j<Batch[i].High; j++)   // anything extra in the Z direction
            {
               Batch[i][j] = 0;   // set the default value to 0
            }
         }

         // 3D arrays

         for (int i=0; i<Data.High; i++)   // everything in the X direction
         {
            for (int j=0; j<Data[i].High; j++)   // everything in the Y direction
            {
               for (int k=OldZ+1; k<Data[i][j].High; k++)   // anything extra in the Z direction
               {
                  Data[i][j][k] = 0;   // set the default value to 0
               }
            }
         }

         for (int i=0; i<WRep.High; i++)   // everything in the X direction
         {
            for (int j=0; j<WRep[i].High; j++)   // everything in the Y direction
            {
               for (int k=OldZ+1; k<WRep[i][j].High; k++)   // anything extra in the Z direction
               {
                  WRep[i][j][k] = 0;   // set the default value to 0
               }
            }
         }

      }

   }
}

// Main calculation loops

void TMainForm::Preparation()
{
   ReadDataFromGridToArray();
   Series1->Clear();
   Memo1->Clear();

  /* Memo1->Lines->Add("                    Robust Analysis of Variance");
   Memo1->Lines->Add("");
   Memo1->Lines->Add("Classical Results: ");
   Memo1->Lines->Add("");   */
   OpenOutFiles();
}

void TMainForm::OpenOutFiles()
{
   AnsiString FileName = "";
   FileName = ProgramDir+"\\Results\\Results.txt";
   output.open(FileName.c_str());

   if (!output)                                                     // check to see if file was opened successfully
   {
      ShowMessage("Cannot open results output file");
   }
   // put in titles
   output << endl << "                                 ROBUST ANALYSIS OF VARIANCE" << endl << endl;
   output << " CLASSICAL ANOVA RESULTS " << endl << endl;
   //output << "                        " << "Geochemical" << "        " << "Sampling" << "        " << "Analysis" << endl;

}

void TMainForm::Calculate()
{
   int nb = 1;   // temporary
   for (int ib=0; ib<nb; ib++)
   {
      for (int i=0; i<nlabs; i++)
      {
         for (int j=0; j<nbatch; j++)
         {
            a = 0;
            for (int k=0; k<nreps; k++)
            {
               a = a + Data[i][j][k];   // sum the analyses for each sample at each site, then calculate mean
            }
            Batch[i][j] = a / nreps;    // line 20
         }
      }

      for (int i=0; i<nlabs; i++)
      {
         a = 0;
         for (int j=0; j<nbatch; j++)
         {
            a = a + Batch[i][j];        // sum the sample means at each site, then calculate mean of that
         }
         Labs[i] = a / nbatch;   // line 30
      }

      a = 0;
      for (int i=0; i<nlabs; i++)       // sum the means at each site
      {
         a = a + Labs[i];   // line 40
      }

      ybar = a / nlabs;                 // calculate the overall mean
      //Memo1->Lines->Add("Classical Results: ");
      //Memo1->Lines->Add("Mean = " + FloatToStrF(ybar,ffGeneral,8,3));   // write mean value to memo

      ClassicalMeanStr = FloatToStrF(ybar,ffGeneral,8,3); //Format("%10.3f",ARRAYOFCONST((ybar)));

      //output << " Mean = " << ClassicalMeanStr.c_str() << endl << endl;
      //output << "                        " << "Geochemical" << "        " << "Sampling" << "        " << "Analysis" << endl;

      s = 0;
      for (int i=0; i<nlabs; i++)
      {
         s = s + pow(Labs[i] - ybar, 2);   // line 50
      }

      ssl = s * nreps * nbatch;
      s = 0;
      for (int i=0; i<nlabs; i++)
      {
         for (int j=0; j<nbatch; j++)
         {
            s = s + pow(Batch[i][j] - Labs[i], 2);   // line 60
         }
      }

      ssb = s * nreps;
      s = 0;
      for (int i=0; i<nlabs; i++)
      {
         for (int j=0; j<nbatch; j++)
         {
            for (int k=0; k<nreps; k++)
            {
               s = s + pow(Data[i][j][k] - Batch[i][j], 2);   // line 70
            }
         }
      }

      sse = s;
      //Memo1->Lines->Add("Sums of Squares = " + FloatToStrF(ssl,ffGeneral,8,3)+",  "+FloatToStrF(ssb,ffGeneral,8,3)+",  "+FloatToStrF(sse,ffGeneral,8,3));
      SumSslStr = FloatToStrF(ssl,ffGeneral,8,3); //Format("%10.3f",ARRAYOFCONST((ssl)));
      do
      {
         SumSslStr = SumSslStr+" ";
      } while (SumSslStr.Length() <= 10);

      SumSsbStr = FloatToStrF(ssb,ffGeneral,8,3); //Format("%10.3f",ARRAYOFCONST((ssb)));
      do
      {
         SumSsbStr = SumSsbStr+" ";
      } while (SumSsbStr.Length() <= 10);

      SumSseStr = FloatToStrF(sse,ffGeneral,8,3); //Format("%10.3f",ARRAYOFCONST((sse)));
      do
      {
         SumSseStr = SumSseStr+" ";
      } while (SumSseStr.Length() <= 10);


      msl = ssl / (nlabs-1);
      msb = ssb / ((nbatch-1) * nlabs);
      mse = sse / ((nreps-1) * nbatch * nlabs);
      se = sqrt(mse);

      if (0 > (msb-mse)) {a = 0;}     // zero is bigger
      else {a = (msb-mse) / nreps;}   // msb-mse is bigger

      sb = sqrt(a);

      if (0 > (msl-msb)) {a = 0;}                // zero is bigger
      else {a = (msl-msb) / (nreps * nbatch);}   // msl-msb is bigger

      sl = sqrt(a);
      //Memo1->Lines->Add("Sigma values (geochem, sampling, analysis) = " + FloatToStrF(sl,ffGeneral,8,3)+",  "+FloatToStrF(sb,ffGeneral,8,3)+",  "+FloatToStrF(se,ffGeneral,8,3));
      SigmaSlStr = FloatToStrF(sl,ffGeneral,8,3); //Format("%10.3f",ARRAYOFCONST((sl)));
      do
      {
         SigmaSlStr = SigmaSlStr+" ";
      } while (SigmaSlStr.Length() <= 10);

      SigmaSbStr = FloatToStrF(sb,ffGeneral,8,3); //Format("%10.3f",ARRAYOFCONST((sb)));
      do
      {
         SigmaSbStr = SigmaSbStr+" ";
      } while (SigmaSbStr.Length() <= 10);

      SigmaSeStr = FloatToStrF(se,ffGeneral,8,3); //Format("%10.3f",ARRAYOFCONST((se)));
      do
      {
         SigmaSeStr = SigmaSeStr+" ";
      } while (SigmaSeStr.Length() <= 10);

      sr = sl * sl + sb * sb + se * se;
      //Memo1->Lines->Add("Percent variance (geochem, sampling, analysis) = " + FloatToStrF(sl*sl/sr*100,ffGeneral,8,3)+",  "+FloatToStrF(sb*sb/sr*100,ffGeneral,8,3)
         //+",  "+FloatToStrF(se*se/sr*100,ffGeneral,8,3));
      VarianceSlStr = FloatToStrF(sl*sl/sr*100,ffGeneral,8,3); //Format("%10.3f",ARRAYOFCONST((sl*sl/sr*100)));
      do
      {
         VarianceSlStr = VarianceSlStr+" ";
      } while (VarianceSlStr.Length() <= 10);

      VarianceSbStr = FloatToStrF(sb*sb/sr*100,ffGeneral,8,3); //Format("%10.3f",ARRAYOFCONST((sb*sb/sr*100)));
      do
      {
         VarianceSbStr = VarianceSbStr+" ";
      } while (VarianceSbStr.Length() <= 10);

      VarianceSeStr = FloatToStrF(se*se/sr*100,ffGeneral,8,3); //Format("%10.3f",ARRAYOFCONST((se*se/sr*100)));
      do
      {
         VarianceSeStr = VarianceSeStr+" ";
      } while (VarianceSeStr.Length() <= 10);

      sr = sqrt(sr);
      //Memo1->Lines->Add("Sigma (total) = " + FloatToStrF(sr,ffGeneral,8,3));
      SigmaTotalStr = FloatToStrF(sr,ffGeneral,8,3); //Format("%10.3f",ARRAYOFCONST((sr)));

      //Memo1->Lines->Add("");
      //Memo1->Lines->Add("Robust Results:");

      // do reps

      Ws.Length = (nlabs*nbatch*nreps);
      I = -1;
      for (int i=0; i<nlabs; i++)
      {
         for (int j=0; j<nbatch; j++)
         {
            for (int k=0; k<nreps; k++)
            {
               I = I + 1;
               Ws[I] = fabs(Data[i][j][k] - Batch[i][j]);   // line 100
            }
         }
      }

      as = Median();
      if (as <= 0)
      {
         as = sqrt(mse);
      }

      float Value1 = nreps-1;
      float Value2 = nreps;
      float Value3 = Value1 / Value2;
      c1 = c * sqrt(Value3);
      do
      {
         as1 = as;
         sum2 = 0;
         for (int i=0; i<nlabs; i++)
         {
            for (int j=0; j<nbatch; j++)
            {
               WBatch[i][j] = 0;
               for (int k=0; k<nreps; k++)
               {
                  WRep[i][j][k] = max(-c1*as, min(c1*as, Data[i][j][k]-Batch[i][j]));
                  WBatch[i][j] = WBatch[i][j] + WRep[i][j][k];
                  sum2 = sum2 + pow(WRep[i][j][k], 2);   // line 130
               }
            }
         }

         float Value4 = beta * (nreps-1) * nbatch * nlabs;
         float Value5 = sum2 / Value4;
         as = sqrt(Value5);
         for (int i=0; i<nlabs; i++)
         {
            for (int j=0; j<nbatch; j++)
            {
               Batch[i][j] = Batch[i][j] + WBatch[i][j] / nreps;
            }
         }
      }
      while (as1 != as);

      mse = as * as;

      // now do batches

      Ws.Length = (nbatch);
      for (int i=0; i<nlabs; i++)
      {
         for (int j=0; j<nbatch; j++)
         {
            Ws[j] = Batch[i][j];   // line 200
         }
         Labs[i] = Median();       // line 210
      }

      Ws.Length = (nlabs*nbatch);
      for (int i=0; i<nlabs; i++)
      {
         for (int j=0; j<nbatch; j++)
         {
            Ws[j+i*nbatch] = fabs(Batch[i][j] - Labs[i]);   // line 220
         }
      }

      as = 1.483 * Median();
      if (as <= 0)
      {
         float Value6 = nreps;
         float Value7 = msb / Value6;
         as = sqrt(Value7);
      }

      float Value17 = nbatch-1;
      float Value18 = nbatch;
      float Value19 = Value17 / Value18;
      c2 = c * sqrt(Value19);

      do
      {
         as1 = as;
         sum2 = 0;
         for (int i=0; i<nlabs; i++)
         {
            WLabs[i] = 0;
            for (int j=0; j<nbatch; j++)
            {
               WBatch[i][j] = max(-c2*as, min(c2*as, Batch[i][j] - Labs[i]));
               WLabs[i] = WLabs[i] + WBatch[i][j];
               sum2 = sum2 + pow(WBatch[i][j], 2);
            }
         }

         for (int i=0; i<nlabs; i++)
         {
            Labs[i] = Labs[i] + WLabs[i] / nbatch;   // line 260
         }

         float Value8 = beta * nlabs * (nbatch-1);
         float Value9 = sum2 / Value8;
         as = sqrt(Value9);
      }
      while (as1 != as);

      msb = nreps * as * as;

      // now do labs

      Ws.Length = nlabs;
      for (int i=0; i<nlabs; i++)
      {
         Ws[i] = fabs(Labs[i]);
      }
      am = Median();

      Ws.Length = nlabs;
      for (int i=0; i<nlabs; i++)
      {
         Ws[i] = fabs(Labs[i] - am);   // line 300
      }

      as = 1.483 * Median();
      if (as <= 0)
      {
         float Value10 = nreps * nbatch;
         float Value11 = msl / Value10;
         as = sqrt(Value11);
      }

      float Value12 = nlabs-1;
      float Value13 = nlabs;
      float Value14 = Value12 / Value13;
      c3 = c * sqrt(Value14);

      do
      {
         as1 = as;
         sum1 = 0;
         sum2 = 0;
         for (int i=0; i<nlabs; i++)
         {
            WLabs[i] = max(-c3*as, min(c3*as, Labs[i]-am));
            sum1 = sum1 + WLabs[i];
            sum2 = sum2 + pow(WLabs[i], 2);
         }

         float Value15 = beta * (nlabs-1);
         float Value16 = sum2 / Value15;
         as = sqrt(Value16);
         am = am + sum1 / nlabs;

         if (fabs(as-as1) / as < 1E-7)
         {
            as1 = as;
         }
      }
      while (as != as1);

      msl = as * as * nreps * nbatch;
      //Memo1->Lines->Add("mean = " + FloatToStrF(am,ffGeneral,8,3));   // mean
      RobustMeanStr = FloatToStrF(am,ffGeneral,8,3); //Format("%10.3f",ARRAYOFCONST((am)));

      se = sqrt(mse);

      if (0 > (msb-mse)) {a = 0;}
      else {a = (msb-mse) / nreps;}

      sb = sqrt(a);

      if (0 > (msl-msb)) {a = 0;}
      else {a = (msl-msb) / (nreps * nbatch);}

      sl = sqrt(a);
      sm = sqrt((sb*sb)+(se*se));

      //Memo1->Lines->Add("Sigma values (geochem, sampling, analysis) = " + FloatToStrF(sl,ffGeneral,8,3)+",  "+FloatToStrF(sb,ffGeneral,8,3)+",  "+FloatToStrF(se,ffGeneral,8,3));   // sigma values
      RobustSlStr = FloatToStrF(sl,ffGeneral,8,3); //Format("%10.3f",ARRAYOFCONST((sl)));
      do
      {
         RobustSlStr = RobustSlStr+" ";
      } while (RobustSlStr.Length() <= 10);

      RobustSbStr = FloatToStrF(sb,ffGeneral,8,3); //Format("%10.3f",ARRAYOFCONST((sb)));
      do
      {
         RobustSbStr = RobustSbStr+" ";
      } while (RobustSbStr.Length() <= 10);

      RobustSeStr = FloatToStrF(se,ffGeneral,8,3); //Format("%10.3f",ARRAYOFCONST((se)));
      do
      {
         RobustSeStr = RobustSeStr+" ";
      } while (RobustSeStr.Length() <= 10);

      RobustSmStr = FloatToStrF(sm,ffGeneral,8,3); //Format("%10.3f",ARRAYOFCONST((sm)));
      do
      {
         RobustSmStr = RobustSmStr+" ";
      } while (RobustSmStr.Length() <= 10);

      sr = sl * sl + sb * sb + se * se;
      //Memo1->Lines->Add("Percent variance (geochem, sampling, analysis) = " + FloatToStrF(sl*sl/sr*100,ffGeneral,8,3)+",  "+FloatToStrF(sb*sb/sr*100,ffGeneral,8,3)
         //+",  "+FloatToStrF(se*se/sr*100,ffGeneral,8,3));   // percent variances
      RobustVarianceSlStr = FloatToStrF(sl*sl/sr*100,ffGeneral,8,3); //Format("%10.3f",ARRAYOFCONST((sl*sl/sr*100)));
      do
      {
         RobustVarianceSlStr = RobustVarianceSlStr+" ";
      } while (RobustVarianceSlStr.Length() <= 10);

      RobustVarianceSbStr = FloatToStrF(sb*sb/sr*100,ffGeneral,8,3); //Format("%10.3f",ARRAYOFCONST((sb*sb/sr*100)));
      do
      {
         RobustVarianceSbStr = RobustVarianceSbStr+" ";
      } while (RobustVarianceSbStr.Length() <= 10);

      RobustVarianceSeStr = FloatToStrF(se*se/sr*100,ffGeneral,8,3); //Format("%10.3f",ARRAYOFCONST((se*se/sr*100)));
      do
      {
         RobustVarianceSeStr = RobustVarianceSeStr+" ";
      } while (RobustVarianceSeStr.Length() <= 10);

      RobustVarianceSmStr = FloatToStrF(sm*sm/sr*100,ffGeneral,8,3); //Format("%10.3f",ARRAYOFCONST((sm*sm/sr*100)));
      do
      {
         RobustVarianceSmStr = RobustVarianceSmStr+" ";
      } while (RobustVarianceSmStr.Length() <= 10);


      sr = sqrt(sr);
      //Memo1->Lines->Add("Sigma (total) = " + FloatToStrF(sr,ffGeneral,8,3));   // sigma total
      RobustSigmaStr = FloatToStrF(sr,ffGeneral,8,3); //Format("%10.3f",ARRAYOFCONST((sr)));
      
      // relative uncertainty

      RobustUncertaintySbStr = FloatToStrF(200*sb/am,ffGeneral,8,3); //Format("%10.3f",ARRAYOFCONST(((200*sb)/am)));
      do
      {
         RobustUncertaintySbStr = RobustUncertaintySbStr+" ";
      } while (RobustUncertaintySbStr.Length() <= 10);

      RobustUncertaintySeStr = FloatToStrF(200*se/am,ffGeneral,8,3); //Format("%10.3f",ARRAYOFCONST(((200*se)/am)));
      do
      {
         RobustUncertaintySeStr = RobustUncertaintySeStr+" ";
      } while (RobustUncertaintySeStr.Length() <= 10);

      RobustUncertaintySmStr = FloatToStrF(200*sm/am,ffGeneral,8,3); //Format("%10.3f",ARRAYOFCONST(((200*sm)/am)));
      do
      {
         RobustUncertaintySmStr = RobustUncertaintySmStr+" ";
      } while (RobustUncertaintySmStr.Length() <= 10);

      sr = sl * sl + sb * sb + se * se;
      
      Series1->Add(sl*sl/sr*100,"Geochemical",clBlue);
      Series1->Add(sb*sb/sr*100,"Sampling",clRed);
      Series1->Add(se*se/sr*100,"Analysis",clYellow);

      WriteToFile();
      CloseOutFiles();

   }
}

// write values to file

void TMainForm::WriteToFile()
{
   output << " Mean = " << ClassicalMeanStr.c_str() << endl;
   output << " Standard Deviation (Total) = " << SigmaTotalStr.c_str() << endl << endl;
   output << "                       " << "Geochemical" << "          " << "Sampling" << "          " << "Analysis" << endl;
   output << "                       " << "-----------" << "          " << "--------" << "          " << "--------" << endl;
   output << " Sums of Squares     " << "   " << SumSslStr.c_str() << "         " << SumSsbStr.c_str() << "       " << SumSseStr.c_str() << endl;
   output << " Standard Deviation  " << "   " << SigmaSlStr.c_str() << "         " << SigmaSbStr.c_str() << "       " << SigmaSeStr.c_str() << endl;
   output << " Percentage Variance " << "   " << VarianceSlStr.c_str() << "         " << VarianceSbStr.c_str() << "       " << VarianceSeStr.c_str() << endl << endl << endl << endl;
   output << " ROBUST ANOVA RESULTS: " << endl << endl;
   output << " Mean = " << RobustMeanStr.c_str() << endl;
   output << " Standard Deviation (Total) = " << RobustSigmaStr.c_str() << endl << endl;
   output << "                       " << "Geochemical" << "          " << "Sampling" << "          " << "Analysis" << "          " << "Measurement" << endl;
   output << "                       " << "-----------" << "          " << "--------" << "          " << "--------" << "          " << "-----------" << endl;
   output << " Standard Deviation    " << " " << RobustSlStr.c_str() << "         " << RobustSbStr.c_str() << "       " << RobustSeStr.c_str() << "       " << RobustSmStr.c_str() << endl;
   output << " Percentage Variance   " << " " << RobustVarianceSlStr.c_str() << "         " << RobustVarianceSbStr.c_str() << "       " << RobustVarianceSeStr.c_str() << "       " << RobustVarianceSmStr.c_str() << endl;
   output << " Relative Uncertainty  " << "     -               " << RobustUncertaintySbStr.c_str() << "       " << RobustUncertaintySeStr.c_str() << "       " << RobustUncertaintySmStr.c_str() << endl;
   output << " (% at 95% confidence) " << endl;

}

// function

float TMainForm::Median()
{
   // bubble sort the Ws array

   bool Done;
   float Temp;
   n = Ws.Length-1;

   do
   {
      Done = true;              // set the flat to true at the start of the loop
      for (int i=0; i<Ws.High; i++)
      {
         if (Ws[i] > Ws[i+1])   // check all values to make sure they are in ascending order
         {
            Temp = Ws[i];       // move Ws 1 into a temporary store
            Ws[i] = Ws[i+1];    // move Ws2 into Ws 1
            Ws[i+1] = Temp;     // move the temp value into Ws2
            Done = false;       // flag that there has been a swap
         }
      }
   }
   while (Done == false);

   int n2;
   n2 = n / 2;
   if (2*n2 == n)
   {
      return median = Ws[n2+1];
   }
   else
   {
      return median = 0.5*(Ws[n2] + Ws[n2+1]);
   }
}

void TMainForm::CloseOutFiles()
{
   output.close();
}
//--------------------------------------------------------------------------

void __fastcall TMainForm::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   int ExitCode = Application->MessageBox("Exit Program?", "Warning",
      MB_YESNO|MB_ICONWARNING);

   switch(ExitCode)
   {
      case IDYES:
         FileCloseClick(Sender);
         CanClose = true;
         break;
      case IDNO:
         CanClose = false;
         break;
   }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FileExitClick(TObject *Sender)
{
   Close();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::HelpButtonClick(TObject *Sender)
{
   ExecuteFile("AcroRd32.exe","RobanHelp.pdf",ProgramDir,SW_SHOW);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FilePrintSetupClick(TObject *Sender)
{
   PrinterSetupDialog1->Execute();        
}
//---------------------------------------------------------------------------


void __fastcall TMainForm::RunButtonClick(TObject *Sender)
{
   bool Ok = true;

   for (int i=1; i<DataGrid->ColCount; i++)
   {
      for (int j=1; j<DataGrid->RowCount; j++)
      {
         if (DataGrid->Cells[i][j] == "")
         {
            Ok = false;
         }
      }
   }
   if (!Ok) {ShowMessage("Incomplete Dataset"); }
   else
   {
      Preparation();
      Calculate();
      PageControl1->ActivePage = ChartSheet;
      RunButton->Enabled = false;
      PageControl1Change(Sender);
      CloseOutFiles();
      Memo1->Lines->LoadFromFile(ProgramDir+"\\Results\\Results.txt");
   }
}
//---------------------------------------------------------------------------
void TMainForm::ReadDataFromGridToArray()
{
  int L=0;                           // column counter
  for (int i=0; i<nlabs; i++)        // loops for reading in the data
  {
     Sites[i] = DataGrid->Cells[0][i+1];   // site number in first column and row i+1 stored in array
     for (int j=0; j<nbatch; j++)
     {
        for (int k=0; k<nreps; k++)
        {
           L=L+1;                    // add one to the column counter
           if (L<=nreps*nbatch)      // if column counter <= number of columns
           {
              Data[i][j][k] = StrToFloat(DataGrid->Cells[L][i+1]);   // data in column number L and row i+1 stored in array
           }
           else                      // if column counter > no. of columns then at start of new row
           {
              L=0;                   // reset column counter to zero
              L=L+1;                 // add one to the column counter
              if (L<=nreps*nbatch)   // repeat previous loop for new row
              {
                 Data[i][j][k] = StrToFloat(DataGrid->Cells[L][i+1]);
              }
           } 
        }
      }
   }
}

void TMainForm::ReadDataFromArrayToGrid()
{
  int L=0;
  for (int i=0; i<nlabs; i++)        // loops for reading in the data
  {
     DataGrid->Cells[0][i+1] = Sites[i];   // site number in first column and row i+1 read into grid
     for (int j=0; j<nbatch; j++)
     {
        for (int k=0; k<nreps; k++)
        {
           L=L+1;                    // add one to the column counter
           if (L<=nreps*nbatch)      // if column counter <= number of columns
           {
              DataGrid->Cells[L][i+1] = FloatToStrF(Data[i][j][k],ffGeneral,6,3);   // data in column number L and row i+1 read into grid
           }
           else                      // if column counter > no. of columns then at start of new row
           {
              L=0;                   // reset column counter to zero
              L=L+1;                 // add one to the column counter
              if (L<=nreps*nbatch)   // repeat previous loop for new row
              {
                 DataGrid->Cells[L][i+1] = FloatToStrF(Data[i][j][k],ffGeneral,6,3);
              }
           }
        }
     }
  }
}

void TMainForm::SaveData()                    // from array to file
{
   ofstream outfile(DataFileName.c_str());     // open output file to write data to

   outfile << RowUpDown->Position << endl;
   outfile << SampleUpDown->Position << endl;
   outfile << AnalysesUpDown->Position << endl;

   for (int i=0; i<nlabs; i++)                 // loops for reading in the data
   {
      outfile << Sites[i].c_str() << " ";
      for (int j=0; j<nbatch; j++)
      {
         for (int k=0; k<nreps; k++)
         {
            outfile << Data[i][j][k] << " " ;   // write values in array to file
         }
      }
      outfile << endl;
   }

   outfile.close();
}

void TMainForm::SaveFile()
{
   ofstream outfile(ExportFileName.c_str());     // open output file to write data to

   for (int i=0; i<nlabs; i++)                 // loops for reading in the data
   {
      outfile << Sites[i].c_str() << " ";
      for (int j=0; j<nbatch; j++)
      {
       for (int k=0; k<nreps; k++)
       {
          outfile << Data[i][j][k] << " " ;   // write values in array to file
       }
    }
    outfile << endl;
  }

  outfile.close();
}

void TMainForm::LoadData()                    // from file to array
{
   ifstream infile(DataFileName.c_str());     // open input file

   int Value;
   infile >> Value;
   RowUpDown->Position = Value;

   int Value2;
   infile >> Value2;
   SampleUpDown->Position = Value2;

   int Value3;
   infile >> Value3;
   AnalysesUpDown->Position = Value3;

   for (int i=0; i<nlabs; i++)                // loops for reading in the data
   {
      string SiteName;
      infile >> SiteName;
      //infile >> Sites[i].c_str();
      Sites[i] = SiteName.c_str();
      cout << Sites[i].c_str();
      for (int j=0; j<nbatch; j++)
      {
         for (int k=0; k<nreps; k++)
         {
            infile >> Data[i][j][k];
            cout << Data[i][j][k];            // write values in file to array
         }
      }
      cout << endl;
   }

   infile.close();

}

void TMainForm::LoadFile()
{
   ifstream infile(ImportFileName.c_str());     // open input file

   for (int i=0; i<nlabs; i++)                  // loops for reading in the data
   {
      string SiteName;
      infile >> SiteName;
      //infile >> Sites[i].c_str();
      Sites[i] = SiteName.c_str();
      cout << Sites[i].c_str();
      for (int j=0; j<nbatch; j++)
      {
         for (int k=0; k<nreps; k++)
         {
            infile >> Data[i][j][k];
            cout << Data[i][j][k];              // write values in file to array
         }
      }
      cout << endl;
   }

   infile.close();

}



void __fastcall TMainForm::FileSaveAsClick(TObject *Sender)
{
   if (PageControl1->ActivePage == DataSheet)                // if the data page is open
   {
      SaveDialog1->Title = "Save Project As";                // set the save dialog box title
      SaveDialog1->Filter = "Project files (*.prj)|*.prj";   // set the files of type box to project file
      SaveDialog1->DefaultExt = String("PRJ");               // set the default extension to prj
      SaveDialog1->InitialDir = ProgramDir+"\\Projects";         // set initial directory to data directory
      SaveDialog1->FileName = "";                            // make sure filename box is empty

      if (SaveDialog1->Execute())                            // if ok button pressed on save dialog box
      {
         DataFileName = SaveDialog1->FileName;               // remember the filename
         MainForm->Caption = "ROBAN - "+ExtractFileName(SaveDialog1->FileName);   // put the file name on the caption bar
         ReadDataFromGridToArray();                          // read data into array
         SaveData();                                         // save data to file
         Altered = false;
      }
   }

   else if (PageControl1->ActivePage == ResultsSheet)          // if the results page is open
   {
      SaveDialog1->Title = "Save Results As";                // set the save dialog box title
      SaveDialog1->Filter = "Results files (*.rsl)|*.rsl";   // set the files of type box to results file
      SaveDialog1->DefaultExt = String("RSL");               // set the default extension to res
      SaveDialog1->InitialDir = ProgramDir+"\\Results";      // set initial directory to results directory
      SaveDialog1->FileName = "";                            // make sure filename box is empty

      if (SaveDialog1->Execute())
      {
         ResultsFileName = SaveDialog1->FileName;            // remember the filename
         Memo1->Lines->SaveToFile(ResultsFileName);          // save results to file
         Altered = false;
      }
   }

   else                                                      // if the chart page is open
   {
      SaveDialog1->Title = "Save Chart As";                  // set the save dialog box title
      SaveDialog1->Filter = "Bitmap File (*.bmp)|*.bmp";     // set the files of type box to bitmap file
      SaveDialog1->DefaultExt = String("BMP");               // set the default extension to bmp
      SaveDialog1->InitialDir = ProgramDir+"\\Results";      // set initial directory to results directory
      SaveDialog1->FileName = "";                            // make sure filename box is empty

      if (SaveDialog1->Execute())
      {
         ChartFileName = SaveDialog1->FileName;              // remember the filename
         Chart1->SaveToBitmapFile(ChartFileName);            // save chart as bitmap file
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FileSaveClick(TObject *Sender)
{
   if (PageControl1->ActivePage == DataSheet)             // if the data page is open
   {
      if (DataFileName != "")                             // if a file name has already been given
      {
         try
         {
            Screen->Cursor = crHourGlass;
            ReadDataFromGridToArray();                    // read data into array
            SaveData();                                   // save data to file
            Altered = false;
         }
         __finally
         {
            Screen->Cursor = crDefault;
         }
      }
      else                                                // if a file name hasn't already been given
      {
         FileSaveAsClick(Sender);                         // go to file save as
      }
   }

   else if (PageControl1->ActivePage == ResultsSheet)      // if the results page is open
   {
      if (ResultsFileName != "")
      {
         try
         {
            Screen->Cursor = crHourGlass;
            Memo1->Lines->SaveToFile(ResultsFileName);    // save results to file
            Altered = false;
         }
         __finally
         {
            Screen->Cursor = crDefault;
         }
      }
      else                                                // if a file name hasn't already been given
      {
         FileSaveAsClick(Sender);                         // go to file save as
      }
   }

   else
   {
      if (ChartFileName != "")                            // if the chart page is open
      {
         try
         {
            Screen->Cursor = crHourGlass;
            Chart1->SaveToBitmapFile(ChartFileName);      // save results to file
            Altered = false;
         }
         __finally
         {
            Screen->Cursor = crDefault;
         }
      }
      else                                                // if a file name hasn't already been given
      {
         FileSaveAsClick(Sender);                         // go to file save as
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FileOpenClick(TObject *Sender)
{
   SetUpArraySizes(0,0,0);                                   // set the array sizes to zero
   for (int i=0; i<DataGrid->RowCount; i++)                  // for each row
   {
      for (int j=0; j<DataGrid->ColCount; j++)               // and for each column in that row
      {
         DataGrid->Cells[j][i] = " ";                        // clear the values in the cells
      }
   }
   ReadDataFromGridToArray();                                // read data into arrays
   RowEditChange(Sender);
   RowUpDown->Position = 2;                                  // set number of rows
   SampleUpDown->Position = 2;                               // set number of columns
   AnalysesUpDown->Position = 2;
   AnalysesEditChange(Sender);
   Memo1->Clear();                                           // remove the previous results
   Series1->Clear();                                         // remove the previous chart

   if (PageControl1->ActivePage == DataSheet)                // if the data page is open
   {
      OpenDialog1->Title = "Open Project";                   // set the open dialog box title
      OpenDialog1->Filter = "Project files (*.prj)|*.prj";   // set the files of type box to project file
      OpenDialog1->InitialDir = ProgramDir+"\\Projects";     // set the initial directory to data directory
      OpenDialog1->FileName = "";                            // make sure filename box is empty

      if (OpenDialog1->Execute())                            // if ok button pressed on open dialog box
      {
         DataFileName = OpenDialog1->FileName;               // remember the filename
         MainForm->Caption = "ROBAN - "+ExtractFileName(OpenDialog1->FileName);   // put the file name on the caption bar
         LoadData();                                         // load data from file to array
         ReadDataFromArrayToGrid();                          // read data into grid
         Altered = false;
      }
   }

   else if (PageControl1->ActivePage == ResultsSheet)        // if the results page is open
   {
      OpenDialog1->Title = "Open Results";                   // set the open dialog box title
      OpenDialog1->Filter = "Results files (*.rsl)|*.rsl";   // set the files of type box to results file
      OpenDialog1->InitialDir = ProgramDir+"\\Results";      // set the initial directory to results directory
      OpenDialog1->FileName = "";                            // make sure filename box is empty

      if (OpenDialog1->Execute())
      {
         ResultsFileName = OpenDialog1->FileName;            // remember the filename
         Memo1->Lines->LoadFromFile(ResultsFileName);        // open the file
         Altered = false;
      }
   }

   else
   {
      FileNewClick(Sender);
      PageControl1->ActivePage = DataSheet;
      OpenDialog1->Title = "Open Project";                   // set the open dialog box title
      OpenDialog1->Filter = "Project files (*.prj)|*.prj";   // set the files of type box to project file
      OpenDialog1->FileName = "";                            // make sure filename box is empty

      if (OpenDialog1->Execute())                            // if ok button pressed on open dialog box
      {
         DataFileName = OpenDialog1->FileName;               // remember the filename
         LoadData();                                         // load data from file to array
         ReadDataFromArrayToGrid();                          // read data into grid
         Altered = false;
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ToolsImportClick(TObject *Sender)
{
   PageControl1->ActivePage = DataSheet;
   if (PageControl1->ActivePage == DataSheet)             // if the data page is open
   {
      OpenDialog1->Title = "Import Project";              // set the open dialog box title
      OpenDialog1->Filter = "All Files (*.*)|*.*";        // set the files of type box to all files
      OpenDialog1->FileName = "";

      if (OpenDialog1->Execute())                         // if ok button pressed on open dialog box
      {
         ImportFileName = OpenDialog1->FileName;          // remember the filename
         LoadFile();                                      // load data from file to array
         ReadDataFromArrayToGrid();                       // read data into grid
         Altered = false;
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ToolsExportClick(TObject *Sender)
{
   if (PageControl1->ActivePage == DataSheet)             // if the data page is open
   {
      SaveDialog1->Title = "Export Project As";           // set the save dialog box title
      SaveDialog1->Filter = "Text files (*.txt)|*.txt";   // set the files of type box to text file
      SaveDialog1->DefaultExt = String("TXT");            // set the default extension to txt
      SaveDialog1->FileName = "";

      if (SaveDialog1->Execute())                         // if ok button pressed on save dialog box
      {
         ExportFileName = SaveDialog1->FileName;          // remember the filename
         ReadDataFromGridToArray();                       // read data into array
         SaveFile();                                      // save data to file
         Altered = false;
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FileNewClick(TObject *Sender)
{
   SetUpArraySizes(0,0,0);                                // set the array sizes to zero
   for (int i=0; i<DataGrid->RowCount; i++)               // for each row
   {
      for (int j=0; j<DataGrid->ColCount; j++)            // and for each column in that row
      {
         DataGrid->Cells[j][i] = " ";                     // clear the values in the cells
      }
   }
   ReadDataFromGridToArray();                             // read data into arrays
   RowEditChange(Sender);
   RowUpDown->Position = 2;                               // set number of rows
   SampleUpDown->Position = 2;                            // set number of columns
   AnalysesUpDown->Position = 2;
   AnalysesEditChange(Sender);
   DataFileName = "";                                     // remove any saved filenames
   ResultsFileName = "";
   ChartFileName = "";
   Memo1->Clear();                                        // remove the previous results
   Series1->Clear();                                      // remove the previous chart
   PageControl1->ActivePage = DataSheet;                  // make data sheet active page
   MainForm->Caption = "ROBAN";

   // Enable the buttons again

   for (int i=2; i<ToolBar2->ButtonCount-1; i++)
   {
      ToolBar2->Buttons[i]->Enabled = true;
   }

   DataSheet->TabVisible = true;
   ResultsSheet->TabVisible = true;
   ChartSheet->TabVisible = true;
   FileSave->Enabled = true;
   FileSaveAs->Enabled = true;
   FilePrintSetup->Enabled = true;
   FilePrint->Enabled = true;
   FileClose->Enabled = true;
   EditUndo->Enabled = true;
   EditCut->Enabled = true;
   EditCopy->Enabled = true;
   EditPaste->Enabled = true;
   EditSelectAll->Enabled = true;
   RunRun->Enabled = true;
   RunButton->Enabled = true;
   ToolsImport->Enabled = true;
   ToolsExport->Enabled = true;
   //HelpContents->Enabled = true;
   HelpAbout->Enabled = true;

   PageControl1Change(Sender);
   Altered = false;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FilePrintClick(TObject *Sender)
{
   if (PageControl1->ActivePage == ResultsSheet)                      // if the results page is open
   {
      if (PrintDialog1->Execute())                                    // if ok button pressed on print dialog box
      {
         Printer()->Title = "Results";                                // set the title for the output
         Printer()->Canvas->Font = Memo1->Font;                       // set the printer font to the same font as the memo
         int LineHeight = abs(MulDiv(Printer()->Canvas->Font->Size,
            GetDeviceCaps(Printer()->Handle, LOGPIXELSY), 72));       // calculate the line height
         LineHeight *= 1.4;
         int LinesPerPage = (Printer()->PageHeight/LineHeight) - 4;   // calculate how many lines will fit on a page
         int LineCount = 4;                                           // start printing on line 4 to allow for title and top margin
         Printer()->BeginDoc();                                       // start printing and print the title
         TRect rect;
         rect.Top = LineHeight;
         rect.Left = 20;
         rect.Right = Printer()->PageWidth;
         rect.Bottom = LineHeight * 2;
         DrawText(Printer()->Handle, "Results", -1, (RECT*)&rect, DT_CENTER);

         // loop through all the lines and print each one

         for (int i=0; i<Memo1->Lines->Count; i++)
         {
            // when you reach the bottom of the page start a new page and reset line count

            if (LineCount++ == LinesPerPage)
            {
               LineCount = 4;
               Printer()->NewPage();
            }

            String s = Memo1->Lines->Strings[i];                      // get the next string and print it using TextOut()
            Printer()->Canvas->TextOut(0, LineCount * LineHeight, s);
         }

         Printer()->EndDoc();                                         // stop printing
      }
   }

   else if (PageControl1->ActivePage == ChartSheet)                     // if chart page is open
   {
      if (PrintDialog1->Execute())                                      // if ok button pressed on print dialog box
      {
         //Chart1->Print();
         Chart1->PrintLandscape();                                      // print chart as landscape
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::PageControl1Change(TObject *Sender)
{
   if (PageControl1->ActivePage == DataSheet)
   {
      FilePrint->Enabled = false;
      FilePrintSetup->Enabled = false;
      FilePrintButton->Enabled = false;
      EditUndo->Enabled = false;
      EditCut->Enabled = false;
      EditCopy->Enabled = false;
      EditPaste->Enabled = false;
      EditSelectAll->Enabled = false;
      ToolsImport->Enabled = true;
      ToolsExport->Enabled = true;
      //MainForm->BorderStyle = bsDialog;
      //MainForm->Constraints->MaxHeight = 540;
      //MainForm->Constraints->MaxWidth = 671;
      FileOpen->Enabled = true;
      FileOpenButton->Enabled = true;
      ToolsChartOptions->Enabled = false;
      ChartOptionsButton->Enabled = false;
      RunButton->Enabled = true;
   }

   else if (PageControl1->ActivePage == ResultsSheet)
   {
      FilePrint->Enabled = true;
      FilePrintSetup->Enabled = true;
      FilePrintButton->Enabled = true;
      EditUndo->Enabled = true;
      EditCut->Enabled = true;
      EditCopy->Enabled = true;
      EditPaste->Enabled = true;
      EditSelectAll->Enabled = true;
      ToolsImport->Enabled = false;
      ToolsExport->Enabled = false;
      //MainForm->BorderStyle = bsSizeable;
      //MainForm->Constraints->MaxHeight = 0;
      //MainForm->Constraints->MaxWidth = 0;
      FileOpen->Enabled = true;
      FileOpenButton->Enabled = true;
      ToolsChartOptions->Enabled = false;
      ChartOptionsButton->Enabled = false;
   }

   else
   {
      FilePrint->Enabled = true;
      FilePrintSetup->Enabled = true;
      FilePrintButton->Enabled = true;
      EditUndo->Enabled = false;
      EditCut->Enabled = false;
      EditCopy->Enabled = false;
      EditPaste->Enabled = false;
      EditSelectAll->Enabled = false;
      ToolsImport->Enabled = false;
      ToolsExport->Enabled = false;
      //MainForm->BorderStyle = bsSizeable;
      //MainForm->Constraints->MaxHeight = 0;
      //MainForm->Constraints->MaxWidth = 0;
      FileOpen->Enabled = false;
      FileOpenButton->Enabled = false;
      ToolsChartOptions->Enabled = true;
      ChartOptionsButton->Enabled = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FileCloseClick(TObject *Sender)
{
   PageControl1->ActivePage = DataSheet;
   if (Altered == true)
   {
      int ExitCode = Application->MessageBox("Save Changes to the Model?", "Warning",
         MB_YESNO|MB_ICONWARNING);

      switch(ExitCode)
      {
         case IDYES:
            FileSaveClick(Sender);
            break;
      }
   }

   for (int i=2; i<ToolBar2->ButtonCount-1; i++)
   {
      ToolBar2->Buttons[i]->Enabled = false;
   }
   DataSheet->TabVisible = false;
   ResultsSheet->TabVisible = false;
   ChartSheet->TabVisible = false;

   FileNewButton->Enabled = true;
   FileOpenButton->Enabled = true;
   FileNew->Enabled = true;
   FileOpen->Enabled = true;
   FileSave->Enabled = false;
   FileSaveAs->Enabled = false;
   FilePrintSetup->Enabled = false;
   FilePrint->Enabled = false;
   FileClose->Enabled = false;
   EditUndo->Enabled = false;
   EditCut->Enabled = false;
   EditCopy->Enabled = false;
   EditPaste->Enabled = false;
   EditSelectAll->Enabled = false;
   RunRun->Enabled = false;
   RunButton->Enabled = false;
   ToolsImport->Enabled = false;
   ToolsExport->Enabled = false;
   HelpContents->Enabled = false;
   HelpAbout->Enabled = true;
   ToolsChartOptions->Enabled = false;
   ChartOptionsButton->Enabled = false;

   SetUpArraySizes(0,0,0);
   DataFileName = "";
   ResultsFileName = "";
   ChartFileName = "";
   Memo1->Clear();
   Series1->Clear();
   MainForm->Caption = "ROBAN";
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::HelpAboutClick(TObject *Sender)
{
   AboutBox->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::EditUndoClick(TObject *Sender)
{
   SendMessage(Memo1->Handle, WM_UNDO, 0, 0);        
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::EditCutClick(TObject *Sender)
{
   Memo1->CutToClipboard();        
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::EditCopyClick(TObject *Sender)
{
   Memo1->CopyToClipboard();        
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::EditPasteClick(TObject *Sender)
{
   Memo1->PasteFromClipboard();        
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::EditSelectAllClick(TObject *Sender)
{
   Memo1->SelectAll();        
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ChartOptionsButtonClick(TObject *Sender)
{
   PieChartOptionsForm->Show();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::DataGridSelectCell(TObject *Sender, int ACol,
      int ARow, bool &CanSelect)
{
   Altered = true;        
}
//---------------------------------------------------------------------------


void __fastcall TMainForm::Timer1Timer(TObject *Sender)
{
   Timer1->Enabled = false;   // turn off timer
   ControlBar1->Enabled = true;
   PageControl1->Enabled = true;
   for (int i=0; i<ToolBar2->ButtonCount; i++)
   {
      ToolBar2->Buttons[i]->Enabled = true;
   }
   PageControl1Change(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::SetProportionofOutliers1Click(TObject *Sender)
{
  AnsiString NewString = FloatToStr(c);
  if (InputQuery("Set Proportion of Outliers", "Proportion of Outliers (x standard deviation)", NewString))
  {
    c = StrToFloat(NewString);
  }
  // C = StrToFloat(InputQuery("Set Proportion of Outliers", "Proportion of Outliers (x standard deviation)", "1.5"));
}
//---------------------------------------------------------------------------

